@@ -27,6 +27,16 @@ pub struct ComboBox {
 }
 
 impl ComboBox {
+    /// Create new `ComboBox` with id and label
+    pub fn new(id_source: impl std::hash::Hash, label: impl Into<WidgetText>) -> Self {
+        Self {
+            id_source: Id::new(id_source),
+            label: Some(label.into()),
+            selected_text: Default::default(),
+            width: None,
+        }
+    }
+
     /// Label shown next to the combo box
     pub fn from_label(label: impl Into<WidgetText>) -> Self {
         let label = label.into();
@@ -196,6 +206,7 @@ fn combo_box_dyn<'c, R>(
         ScrollArea::vertical()
             .max_height(ui.spacing().combo_height)
             .show(ui, menu_contents)
+            .inner
     });
 
     InnerResponse {
@@ -239,7 +250,7 @@ fn button_frame(
             where_to_put_background,
             epaint::RectShape {
                 rect: outer_rect.expand(visuals.expansion),
-                corner_radius: visuals.corner_radius,
+                rounding: visuals.rounding,
                 fill: visuals.bg_fill,
                 stroke: visuals.bg_stroke,
             },
