@@ -1,6 +1,6 @@
 //! Frame container
 
-use crate::{layers::ShapeIdx, *};
+use crate::{layers::ShapeIdx, style::Margin, *};
 use epaint::*;
 
 /// Color and margin of a rectangular background of a [`Ui`].
@@ -8,8 +8,8 @@ use epaint::*;
 #[must_use = "You should call .show()"]
 pub struct Frame {
     /// On each side
-    pub margin: Vec2,
-    pub corner_radius: f32,
+    pub margin: Margin,
+    pub rounding: Rounding,
     pub shadow: Shadow,
     pub fill: Color32,
     pub stroke: Stroke,
@@ -23,8 +23,8 @@ impl Frame {
     /// For when you want to group a few widgets together within a frame.
     pub fn group(style: &Style) -> Self {
         Self {
-            margin: Vec2::splat(6.0), // symmetric looks best in corners when nesting
-            corner_radius: style.visuals.widgets.noninteractive.corner_radius,
+            margin: Margin::same(6.0), // symmetric looks best in corners when nesting
+            rounding: style.visuals.widgets.noninteractive.rounding,
             stroke: style.visuals.widgets.noninteractive.bg_stroke,
             ..Default::default()
         }
@@ -32,8 +32,8 @@ impl Frame {
 
     pub(crate) fn side_top_panel(style: &Style) -> Self {
         Self {
-            margin: Vec2::new(8.0, 2.0),
-            corner_radius: 0.0,
+            margin: Margin::symmetric(8.0, 2.0),
+            rounding: Rounding::none(),
             fill: style.visuals.window_fill(),
             stroke: style.visuals.window_stroke(),
             ..Default::default()
@@ -42,8 +42,8 @@ impl Frame {
 
     pub(crate) fn central_panel(style: &Style) -> Self {
         Self {
-            margin: Vec2::new(8.0, 8.0),
-            corner_radius: 0.0,
+            margin: Margin::symmetric(8.0, 8.0),
+            rounding: Rounding::none(),
             fill: style.visuals.window_fill(),
             stroke: Default::default(),
             ..Default::default()
@@ -52,8 +52,8 @@ impl Frame {
 
     pub fn window(style: &Style) -> Self {
         Self {
-            margin: style.spacing.window_padding,
-            corner_radius: style.visuals.window_corner_radius,
+            margin: style.spacing.window_margin,
+            rounding: style.visuals.window_rounding,
             shadow: style.visuals.window_shadow,
             fill: style.visuals.window_fill(),
             stroke: style.visuals.window_stroke(),
@@ -62,8 +62,8 @@ impl Frame {
 
     pub fn menu(style: &Style) -> Self {
         Self {
-            margin: Vec2::splat(1.0),
-            corner_radius: style.visuals.widgets.noninteractive.corner_radius,
+            margin: Margin::same(1.0),
+            rounding: style.visuals.widgets.noninteractive.rounding,
             shadow: style.visuals.popup_shadow,
             fill: style.visuals.window_fill(),
             stroke: style.visuals.window_stroke(),
@@ -72,8 +72,8 @@ impl Frame {
 
     pub fn popup(style: &Style) -> Self {
         Self {
-            margin: style.spacing.window_padding,
-            corner_radius: style.visuals.widgets.noninteractive.corner_radius,
+            margin: style.spacing.window_margin,
+            rounding: style.visuals.widgets.noninteractive.rounding,
             shadow: style.visuals.popup_shadow,
             fill: style.visuals.window_fill(),
             stroke: style.visuals.window_stroke(),
@@ -83,8 +83,8 @@ impl Frame {
     /// dark canvas to draw on
     pub fn dark_canvas(style: &Style) -> Self {
         Self {
-            margin: Vec2::new(10.0, 10.0),
-            corner_radius: style.visuals.widgets.noninteractive.corner_radius,
+            margin: Margin::symmetric(10.0, 10.0),
+            rounding: style.visuals.widgets.noninteractive.rounding,
             fill: Color32::from_black_alpha(250),
             stroke: style.visuals.window_stroke(),
             ..Default::default()
@@ -103,13 +103,13 @@ impl Frame {
         self
     }
 
-    pub fn corner_radius(mut self, corner_radius: f32) -> Self {
-        self.corner_radius = corner_radius;
+    pub fn rounding(mut self, rounding: impl Into<Rounding>) -> Self {
+        self.rounding = rounding.into();
         self
     }
 
     /// Margin on each side of the frame.
-    pub fn margin(mut self, margin: impl Into<Vec2>) -> Self {
+    pub fn margin(mut self, margin: impl Into<Margin>) -> Self {
         self.margin = margin.into();
         self
     }
@@ -137,7 +137,10 @@ impl Frame {
     pub fn begin(self, ui: &mut Ui) -> Prepared {
         let where_to_put_background = ui.painter().add(Shape::Noop);
         let outer_rect_bounds = ui.available_rect_before_wrap();
-        let mut inner_rect = outer_rect_bounds.shrink2(self.margin);
+
+        let mut inner_rect = outer_rect_bounds;
+        inner_rect.min += Vec2::new(self.margin.left, self.margin.top);
+        inner_rect.max -= Vec2::new(self.margin.right, self.margin.bottom);
 
         // Make sure we don't shrink to the negative:
         inner_rect.max.x = inner_rect.max.x.max(inner_rect.min.x);
@@ -172,7 +175,7 @@ impl Frame {
     pub fn paint(&self, outer_rect: Rect) -> Shape {
         let Self {
             margin: _,
-            corner_radius,
+            rounding,
             shadow,
             fill,
             stroke,
@@ -180,7 +183,7 @@ impl Frame {
 
         let frame_shape = Shape::Rect(epaint::RectShape {
             rect: outer_rect,
-            corner_radius,
+            rounding,
             fill,
             stroke,
         });
@@ -188,7 +191,7 @@ impl Frame {
         if shadow == Default::default() {
             frame_shape
         } else {
-            let shadow = shadow.tessellate(outer_rect, corner_radius);
+            let shadow = shadow.tessellate(outer_rect, rounding);
             let shadow = Shape::Mesh(shadow);
             Shape::Vec(vec![shadow, frame_shape])
         }
@@ -197,7 +200,10 @@ impl Frame {
 
 impl Prepared {
     pub fn outer_rect(&self) -> Rect {
-        self.content_ui.min_rect().expand2(self.frame.margin)
+        let mut rect = self.content_ui.min_rect();
+        rect.min -= Vec2::new(self.frame.margin.left, self.frame.margin.top);
+        rect.max += Vec2::new(self.frame.margin.right, self.frame.margin.bottom);
+        rect
     }
 
     pub fn end(self, ui: &mut Ui) -> Response {
