@@ -14,7 +14,7 @@ pub(crate) mod window;
 
 pub use {
     area::Area,
-    collapsing_header::*,
+    collapsing_header::{CollapsingHeader, CollapsingResponse},
     combo_box::*,
     frame::Frame,
     panel::{CentralPanel, SidePanel, TopBottomPanel},
@@ -23,6 +23,3 @@ pub use {
     scroll_area::ScrollArea,
     window::Window,
 };
-
-#[allow(deprecated)]
-pub use panel::TopPanel;
