@@ -28,6 +28,13 @@ pub struct RawInput {
     /// Set this the first frame, whenever it changes, or just on every frame.
     pub pixels_per_point: Option<f32>,
 
+    /// Maximum size of one side of the font texture.
+    ///
+    /// Ask your graphics drivers about this. This corresponds to `GL_MAX_TEXTURE_SIZE`.
+    ///
+    /// The default is a very small (but very portable) 2048.
+    pub max_texture_side: Option<usize>,
+
     /// Monotonically increasing time, in seconds. Relative to whatever. Used for animations.
     /// If `None` is provided, egui will assume a time delta of `predicted_dt` (default 1/60 seconds).
     pub time: Option<f64>,
@@ -62,6 +69,7 @@ impl Default for RawInput {
         Self {
             screen_rect: None,
             pixels_per_point: None,
+            max_texture_side: None,
             time: None,
             predicted_dt: 1.0 / 60.0,
             modifiers: Modifiers::default(),
@@ -81,6 +89,7 @@ impl RawInput {
         RawInput {
             screen_rect: self.screen_rect.take(),
             pixels_per_point: self.pixels_per_point.take(),
+            max_texture_side: self.max_texture_side.take(),
             time: self.time.take(),
             predicted_dt: self.predicted_dt,
             modifiers: self.modifiers,
@@ -95,6 +104,7 @@ impl RawInput {
         let Self {
             screen_rect,
             pixels_per_point,
+            max_texture_side,
             time,
             predicted_dt,
             modifiers,
@@ -105,6 +115,7 @@ impl RawInput {
 
         self.screen_rect = screen_rect.or(self.screen_rect);
         self.pixels_per_point = pixels_per_point.or(self.pixels_per_point);
+        self.max_texture_side = max_texture_side.or(self.max_texture_side);
         self.time = time; // use latest time
         self.predicted_dt = predicted_dt; // use latest dt
         self.modifiers = modifiers; // use latest
@@ -135,7 +146,7 @@ pub struct DroppedFile {
     /// Set by the `egui_web` backend.
     pub last_modified: Option<std::time::SystemTime>,
     /// Set by the `egui_web` backend.
-    pub bytes: Option<std::sync::Arc<[u8]>>,
+    pub bytes: Option<epaint::mutex::Arc<[u8]>>,
 }
 
 /// An input event generated by the integration.
@@ -148,22 +159,33 @@ pub enum Event {
     Copy,
     /// The integration detected a "cut" event (e.g. Cmd+X).
     Cut,
-    /// Text input, e.g. via keyboard or paste action.
+    /// The integration detected a "paste" event (e.g. Cmd+V).
+    Paste(String),
+    /// Text input, e.g. via keyboard.
     ///
     /// When the user presses enter/return, do not send a `Text` (just [`Key::Enter`]).
     Text(String),
+    /// A key was pressed or released.
     Key {
         key: Key,
+        /// Was it pressed or released?
         pressed: bool,
+        /// The state of the modifier keys at the time of the event.
         modifiers: Modifiers,
     },
 
+    /// The mouse or touch moved to a new place.
     PointerMoved(Pos2),
+
+    /// A mouse button was pressed or released (or a touch started or stopped).
     PointerButton {
+        /// Where is the pointer?
         pos: Pos2,
+        /// What mouse button? For touches, use [`PointerButton::Primary`].
         button: PointerButton,
+        /// Was it the button/touch pressed this frame, or released?
         pressed: bool,
-        /// The state of the modifier keys at the time of the event
+        /// The state of the modifier keys at the time of the event.
         modifiers: Modifiers,
     },
     /// The mouse left the screen, or the last/primary touch input disappeared.
@@ -174,6 +196,12 @@ pub enum Event {
     PointerGone,
 
     /// How many points (logical pixels) the user scrolled.
+    ///
+    /// The direction of the vector indicates how to move the _content_ that is being viewed.
+    /// So if you get positive values, the content being viewed should move to the right and down,
+    /// revealing new things to the left and up.
+    ///
+    /// Shift-scroll should result in horizontal scrolling (it is up to the integrations to do this).
     Scroll(Vec2),
 
     /// Zoom scale factor this frame (e.g. from ctrl-scroll or pinch gesture).
@@ -198,6 +226,7 @@ pub enum Event {
         /// Unique identifier of a finger/pen. Value is stable from touch down
         /// to lift-up
         id: TouchId,
+        /// One of: start move end cancel.
         phase: TouchPhase,
         /// Position of the touch (or where the touch was last detected)
         pos: Pos2,
@@ -225,18 +254,24 @@ pub enum PointerButton {
 pub const NUM_POINTER_BUTTONS: usize = 3;
 
 /// State of the modifier keys. These must be fed to egui.
-#[derive(Clone, Copy, Debug, Default, Eq, PartialEq)]
+///
+/// The best way to compare `Modifiers` is by using [`Modifiers::matches`].
+#[derive(Clone, Copy, Debug, Default, PartialEq)]
 #[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct Modifiers {
     /// Either of the alt keys are down (option ⌥ on Mac).
     pub alt: bool,
+
     /// Either of the control keys are down.
     /// When checking for keyboard shortcuts, consider using [`Self::command`] instead.
     pub ctrl: bool,
+
     /// Either of the shift keys are down.
     pub shift: bool,
+
     /// The Mac ⌘ Command key. Should always be set to `false` on other platforms.
     pub mac_cmd: bool,
+
     /// On Windows and Linux, set this to the same value as `ctrl`.
     /// On Mac, this should be set whenever one of the ⌘ Command keys are down (same as `mac_cmd`).
     /// This is so that egui can, for instance, select all text by checking for `command + A`
@@ -245,6 +280,56 @@ pub struct Modifiers {
 }
 
 impl Modifiers {
+    pub fn new() -> Self {
+        Default::default()
+    }
+
+    pub const NONE: Self = Self {
+        alt: false,
+        ctrl: false,
+        shift: false,
+        mac_cmd: false,
+        command: false,
+    };
+
+    pub const ALT: Self = Self {
+        alt: true,
+        ctrl: false,
+        shift: false,
+        mac_cmd: false,
+        command: false,
+    };
+    pub const CTRL: Self = Self {
+        alt: false,
+        ctrl: true,
+        shift: false,
+        mac_cmd: false,
+        command: false,
+    };
+    pub const SHIFT: Self = Self {
+        alt: false,
+        ctrl: false,
+        shift: true,
+        mac_cmd: false,
+        command: false,
+    };
+    /// The Mac ⌘ Command key
+    pub const MAC_CMD: Self = Self {
+        alt: false,
+        ctrl: false,
+        shift: false,
+        mac_cmd: true,
+        command: false,
+    };
+    /// On Mac: ⌘ Command key, elsewhere: Ctrl key
+    pub const COMMAND: Self = Self {
+        alt: false,
+        ctrl: false,
+        shift: false,
+        mac_cmd: false,
+        command: true,
+    };
+
     #[inline(always)]
     pub fn is_none(&self) -> bool {
         self == &Self::default()
@@ -255,6 +340,7 @@ impl Modifiers {
         !self.is_none()
     }
 
+    /// Is shift the only pressed button?
     #[inline(always)]
     pub fn shift_only(&self) -> bool {
         self.shift && !(self.alt || self.command)
@@ -265,6 +351,66 @@ impl Modifiers {
     pub fn command_only(&self) -> bool {
         !self.alt && !self.shift && self.command
     }
+
+    /// Check for equality but with proper handling of [`Self::command`].
+    ///
+    /// ```
+    /// # use egui::Modifiers;
+    /// assert!(Modifiers::CTRL.matches(Modifiers::CTRL));
+    /// assert!(!Modifiers::CTRL.matches(Modifiers::CTRL | Modifiers::SHIFT));
+    /// assert!(!(Modifiers::CTRL | Modifiers::SHIFT).matches(Modifiers::CTRL));
+    /// assert!((Modifiers::CTRL | Modifiers::COMMAND).matches(Modifiers::CTRL));
+    /// assert!((Modifiers::CTRL | Modifiers::COMMAND).matches(Modifiers::COMMAND));
+    /// assert!((Modifiers::MAC_CMD | Modifiers::COMMAND).matches(Modifiers::COMMAND));
+    /// assert!(!Modifiers::COMMAND.matches(Modifiers::MAC_CMD));
+    /// ```
+    pub fn matches(&self, pattern: Modifiers) -> bool {
+        // alt and shift must always match the pattern:
+        if pattern.alt != self.alt || pattern.shift != self.shift {
+            return false;
+        }
+
+        if pattern.mac_cmd {
+            // Mac-specific match:
+            if !self.mac_cmd {
+                return false;
+            }
+            if pattern.ctrl != self.ctrl {
+                return false;
+            }
+            return true;
+        }
+
+        if !pattern.ctrl && !pattern.command {
+            // the pattern explicitly doesn't want any ctrl/command:
+            return !self.ctrl && !self.command;
+        }
+
+        // if the pattern is looking for command, then `ctrl` may or may not be set depending on platform.
+        // if the pattern is looking for `ctrl`, then `command` may or may not be set depending on platform.
+
+        if pattern.ctrl && !self.ctrl {
+            return false;
+        }
+        if pattern.command && !self.command {
+            return false;
+        }
+
+        true
+    }
+}
+
+impl std::ops::BitOr for Modifiers {
+    type Output = Self;
+    fn bitor(self, rhs: Self) -> Self {
+        Self {
+            alt: self.alt | rhs.alt,
+            ctrl: self.ctrl | rhs.ctrl,
+            shift: self.shift | rhs.shift,
+            mac_cmd: self.mac_cmd | rhs.mac_cmd,
+            command: self.command | rhs.command,
+        }
+    }
 }
 
 /// Keyboard keys.
@@ -349,6 +495,7 @@ impl RawInput {
         let Self {
             screen_rect,
             pixels_per_point,
+            max_texture_side,
             time,
             predicted_dt,
             modifiers,
@@ -362,6 +509,7 @@ impl RawInput {
             .on_hover_text(
                 "Also called HDPI factor.\nNumber of physical pixels per each logical pixel.",
             );
+        ui.label(format!("max_texture_side: {:?}", max_texture_side));
         if let Some(time) = time {
             ui.label(format!("time: {:.3} s", time));
         } else {
@@ -369,10 +517,13 @@ impl RawInput {
         }
         ui.label(format!("predicted_dt: {:.1} ms", 1e3 * predicted_dt));
         ui.label(format!("modifiers: {:#?}", modifiers));
-        ui.label(format!("events: {:?}", events))
-            .on_hover_text("key presses etc");
         ui.label(format!("hovered_files: {}", hovered_files.len()));
         ui.label(format!("dropped_files: {}", dropped_files.len()));
+        ui.scope(|ui| {
+            ui.set_min_height(150.0);
+            ui.label(format!("events: {:#?}", events))
+                .on_hover_text("key presses etc");
+        });
     }
 }
 
