@@ -1,12 +1,11 @@
 // #![warn(missing_docs)]
 
-use epaint::mutex::RwLock;
+use epaint::mutex::{Arc, RwLock, RwLockReadGuard, RwLockWriteGuard};
 use std::hash::Hash;
-use std::sync::Arc;
 
 use crate::{
-    color::*, containers::*, epaint::text::Fonts, layout::*, menu::MenuState, mutex::MutexGuard,
-    placer::Placer, widgets::*, *,
+    color::*, containers::*, epaint::text::Fonts, layout::*, menu::MenuState, placer::Placer,
+    widgets::*, *,
 };
 
 // ----------------------------------------------------------------------------
@@ -49,7 +48,7 @@ pub struct Ui {
     /// The `Style` (visuals, spacing, etc) of this ui.
     /// Commonly many `Ui`:s share the same `Style`.
     /// The `Ui` implements copy-on-write for this.
-    style: std::sync::Arc<Style>,
+    style: Arc<Style>,
 
     /// Handles the `Ui` size and the placement of new widgets.
     placer: Placer,
@@ -70,7 +69,7 @@ impl Ui {
     ///
     /// Normally you would not use this directly, but instead use
     /// [`SidePanel`], [`TopBottomPanel`], [`CentralPanel`], [`Window`] or [`Area`].
-    pub fn new(ctx: CtxRef, layer_id: LayerId, id: Id, max_rect: Rect, clip_rect: Rect) -> Self {
+    pub fn new(ctx: Context, layer_id: LayerId, id: Id, max_rect: Rect, clip_rect: Rect) -> Self {
         let style = ctx.style();
         Ui {
             id,
@@ -122,7 +121,7 @@ impl Ui {
     ///
     /// Note that this may be a different [`Style`] than that of [`Context::style`].
     #[inline]
-    pub fn style(&self) -> &std::sync::Arc<Style> {
+    pub fn style(&self) -> &Arc<Style> {
         &self.style
     }
 
@@ -134,17 +133,17 @@ impl Ui {
     /// Example:
     /// ```
     /// # egui::__run_test_ui(|ui| {
-    /// ui.style_mut().body_text_style = egui::TextStyle::Heading;
+    /// ui.style_mut().override_text_style = Some(egui::TextStyle::Heading);
     /// # });
     /// ```
     pub fn style_mut(&mut self) -> &mut Style {
-        std::sync::Arc::make_mut(&mut self.style) // clone-on-write
+        Arc::make_mut(&mut self.style) // clone-on-write
     }
 
     /// Changes apply to this `Ui` and its subsequent children.
     ///
     /// To set the visuals of all `Ui`:s, use [`Context::set_visuals`].
-    pub fn set_style(&mut self, style: impl Into<std::sync::Arc<Style>>) {
+    pub fn set_style(&mut self, style: impl Into<Arc<Style>>) {
         self.style = style.into();
     }
 
@@ -195,9 +194,9 @@ impl Ui {
         &mut self.style_mut().visuals
     }
 
-    /// Get a reference to the parent [`CtxRef`].
+    /// Get a reference to the parent [`Context`].
     #[inline]
-    pub fn ctx(&self) -> &CtxRef {
+    pub fn ctx(&self) -> &Context {
         self.painter.ctx()
     }
 
@@ -236,7 +235,7 @@ impl Ui {
     /// ```
     pub fn set_enabled(&mut self, enabled: bool) {
         self.enabled &= enabled;
-        if !self.enabled && self.visible() {
+        if !self.enabled && self.is_visible() {
             self.painter
                 .set_fade_to_color(Some(self.visuals().window_fill()));
         }
@@ -244,8 +243,13 @@ impl Ui {
 
     /// If `false`, any widgets added to the `Ui` will be invisible and non-interactive.
     #[inline]
+    pub fn is_visible(&self) -> bool {
+        self.painter.is_visible()
+    }
+
+    #[deprecated = "Renamed is_visible"]
     pub fn visible(&self) -> bool {
-        self.painter.visible()
+        self.painter.is_visible()
     }
 
     /// Calling `set_visible(false)` will cause all further widgets to be invisible,
@@ -308,34 +312,79 @@ impl Ui {
         self.painter().layer_id()
     }
 
-    /// The `Input` of the `Context` associated with the `Ui`.
+    /// The [`InputState`] of the [`Context`] associated with this [`Ui`].
     /// Equivalent to `.ctx().input()`.
+    ///
+    /// Note that this locks the [`Context`], so be careful with if-let bindings:
+    ///
+    /// ```
+    /// # egui::__run_test_ui(|ui| {
+    /// if let Some(pos) = { ui.input().pointer.hover_pos() } {
+    ///     // This is fine!
+    /// }
+    ///
+    /// let pos = ui.input().pointer.hover_pos();
+    /// if let Some(pos) = pos {
+    ///     // This is also fine!
+    /// }
+    ///
+    /// if let Some(pos) = ui.input().pointer.hover_pos() {
+    ///     // ⚠️ Using `ui` again here will lead to a dead-lock!
+    /// }
+    /// # });
+    /// ```
     #[inline]
-    pub fn input(&self) -> &InputState {
+    pub fn input(&self) -> RwLockReadGuard<'_, InputState> {
         self.ctx().input()
     }
 
-    /// The `Memory` of the `Context` associated with the `Ui`.
+    /// The [`InputState`] of the [`Context`] associated with this [`Ui`].
+    /// Equivalent to `.ctx().input_mut()`.
+    ///
+    /// Note that this locks the [`Context`], so be careful with if-let bindings
+    /// like for [`Self::input()`].
+    /// ```
+    /// # egui::__run_test_ui(|ui| {
+    /// ui.input_mut().consume_key(egui::Modifiers::default(), egui::Key::Enter);
+    /// # });
+    /// ```
+    #[inline]
+    pub fn input_mut(&self) -> RwLockWriteGuard<'_, InputState> {
+        self.ctx().input_mut()
+    }
+
+    /// The [`Memory`] of the [`Context`] associated with this ui.
     /// Equivalent to `.ctx().memory()`.
     #[inline]
-    pub fn memory(&self) -> MutexGuard<'_, Memory> {
+    pub fn memory(&self) -> RwLockWriteGuard<'_, Memory> {
         self.ctx().memory()
     }
 
-    /// The `Output` of the `Context` associated with the `Ui`.
+    /// Stores superficial widget state.
+    #[inline]
+    pub fn data(&self) -> RwLockWriteGuard<'_, crate::util::IdTypeMap> {
+        self.ctx().data()
+    }
+
+    /// The [`PlatformOutput`] of the [`Context`] associated with this ui.
     /// Equivalent to `.ctx().output()`.
     #[inline]
-    pub fn output(&self) -> MutexGuard<'_, Output> {
+    pub fn output(&self) -> RwLockWriteGuard<'_, PlatformOutput> {
         self.ctx().output()
     }
 
-    /// The `Fonts` of the `Context` associated with the `Ui`.
+    /// The [`Fonts`] of the [`Context`] associated with this ui.
     /// Equivalent to `.ctx().fonts()`.
     #[inline]
-    pub fn fonts(&self) -> &Fonts {
+    pub fn fonts(&self) -> RwLockReadGuard<'_, Fonts> {
         self.ctx().fonts()
     }
 
+    /// The height of text of this text style
+    pub fn text_style_height(&self, style: &TextStyle) -> f32 {
+        self.fonts().row_height(&style.resolve(self.style()))
+    }
+
     /// Screen-space rectangle for clipping what we paint in this ui.
     /// This is used, for instance, to avoid painting outside a window that is smaller than its contents.
     #[inline]
@@ -351,7 +400,7 @@ impl Ui {
 
     /// Can be used for culling: if `false`, then no part of `rect` will be visible on screen.
     pub fn is_rect_visible(&self, rect: Rect) -> bool {
-        self.visible() && rect.intersects(self.clip_rect())
+        self.is_visible() && rect.intersects(self.clip_rect())
     }
 }
 
@@ -855,7 +904,36 @@ impl Ui {
         (response, painter)
     }
 
-    /// Move the scroll to this cursor position with the specified alignment.
+    /// Adjust the scroll position of any parent [`ScrollArea`] so that the given `Rect` becomes visible.
+    ///
+    /// If `align` is `None`, it'll scroll enough to bring the cursor into view.
+    ///
+    /// See also: [`Response::scroll_to_me`], [`Ui::scroll_to_rect`].
+    ///
+    /// ```
+    /// # use egui::Align;
+    /// # egui::__run_test_ui(|ui| {
+    /// egui::ScrollArea::vertical().show(ui, |ui| {
+    ///     // …
+    ///     let response = ui.button("Center on me.");
+    ///     if response.clicked() {
+    ///         ui.scroll_to_rect(response.rect, Some(Align::Center));
+    ///     }
+    /// });
+    /// # });
+    /// ```
+    pub fn scroll_to_rect(&self, rect: Rect, align: Option<Align>) {
+        for d in 0..2 {
+            let range = rect.min[d]..=rect.max[d];
+            self.ctx().frame_state().scroll_target[d] = Some((range, align));
+        }
+    }
+
+    /// Adjust the scroll position of any parent [`ScrollArea`] so that the cursor (where the next widget goes) becomes visible.
+    ///
+    /// If `align` is not provided, it'll scroll enough to bring the cursor into view.
+    ///
+    /// See also: [`Response::scroll_to_me`], [`Ui::scroll_to_rect`].
     ///
     /// ```
     /// # use egui::Align;
@@ -867,15 +945,16 @@ impl Ui {
     ///     }
     ///
     ///     if scroll_bottom {
-    ///         ui.scroll_to_cursor(Align::BOTTOM);
+    ///         ui.scroll_to_cursor(Some(Align::BOTTOM));
     ///     }
     /// });
     /// # });
     /// ```
-    pub fn scroll_to_cursor(&mut self, align: Align) {
+    pub fn scroll_to_cursor(&self, align: Option<Align>) {
         let target = self.next_widget_position();
         for d in 0..2 {
-            self.ctx().frame_state().scroll_target[d] = Some((target[d], align));
+            let target = target[d];
+            self.ctx().frame_state().scroll_target[d] = Some((target..=target, align));
         }
     }
 }
@@ -935,9 +1014,9 @@ impl Ui {
         .inner
     }
 
-    /// Add a  single[`Widget`] that is possibly disabled, i.e. greyed out and non-interactive.
+    /// Add a single [`Widget`] that is possibly disabled, i.e. greyed out and non-interactive.
     ///
-    /// If you call `add_enabled` from within an already disabled UI,
+    /// If you call `add_enabled` from within an already disabled `Ui`,
     /// the widget will always be disabled, even if the `enabled` argument is true.
     ///
     /// See also [`Self::add_enabled_ui`] and [`Self::is_enabled`].
@@ -948,21 +1027,21 @@ impl Ui {
     /// # });
     /// ```
     pub fn add_enabled(&mut self, enabled: bool, widget: impl Widget) -> Response {
-        if enabled || !self.is_enabled() {
-            self.add(widget)
-        } else {
+        if self.is_enabled() && !enabled {
             let old_painter = self.painter.clone();
             self.set_enabled(false);
             let response = self.add(widget);
             self.enabled = true;
             self.painter = old_painter;
             response
+        } else {
+            self.add(widget)
         }
     }
 
     /// Add a section that is possibly disabled, i.e. greyed out and non-interactive.
     ///
-    /// If you call `add_enabled_ui` from within an already disabled UI,
+    /// If you call `add_enabled_ui` from within an already disabled `Ui`,
     /// the result will always be disabled, even if the `enabled` argument is true.
     ///
     /// See also [`Self::add_enabled`] and [`Self::is_enabled`].
@@ -990,10 +1069,67 @@ impl Ui {
         })
     }
 
+    /// Add a single [`Widget`] that is possibly invisible.
+    ///
+    /// An invisible widget still takes up the same space as if it were visible.
+    ///
+    /// If you call `add_visible` from within an already invisible `Ui`,
+    /// the widget will always be invisible, even if the `visible` argument is true.
+    ///
+    /// See also [`Self::add_visible_ui`], [`Self::set_visible`] and [`Self::is_visible`].
+    ///
+    /// ```
+    /// # egui::__run_test_ui(|ui| {
+    /// ui.add_visible(false, egui::Label::new("You won't see me!"));
+    /// # });
+    /// ```
+    pub fn add_visible(&mut self, visible: bool, widget: impl Widget) -> Response {
+        if self.is_visible() && !visible {
+            // temporary make us invisible:
+            let old_painter = self.painter.clone();
+            self.set_visible(false);
+            let response = self.add(widget);
+            self.painter = old_painter;
+            response
+        } else {
+            self.add(widget)
+        }
+    }
+
+    /// Add a section that is possibly invisible, i.e. greyed out and non-interactive.
+    ///
+    /// An invisible ui still takes up the same space as if it were visible.
+    ///
+    /// If you call `add_visible_ui` from within an already invisible `Ui`,
+    /// the result will always be invisible, even if the `visible` argument is true.
+    ///
+    /// See also [`Self::add_visible`], [`Self::set_visible`] and [`Self::is_visible`].
+    ///
+    /// ### Example
+    /// ```
+    /// # egui::__run_test_ui(|ui| {
+    /// # let mut visible = true;
+    /// ui.checkbox(&mut visible, "Show subsection");
+    /// ui.add_visible_ui(visible, |ui| {
+    ///     ui.label("Maybe you see this, maybe you don't!");
+    /// });
+    /// # });
+    /// ```
+    pub fn add_visible_ui<R>(
+        &mut self,
+        visible: bool,
+        add_contents: impl FnOnce(&mut Ui) -> R,
+    ) -> InnerResponse<R> {
+        self.scope(|ui| {
+            ui.set_visible(visible);
+            add_contents(ui)
+        })
+    }
+
     /// Add extra space before the next widget.
     ///
     /// The direction is dependent on the layout.
-    /// This will be in addition to the [`Spacing::item_spacing`}.
+    /// This will be in addition to the [`crate::style::Spacing::item_spacing`].
     ///
     /// [`Self::min_rect`] will expand to contain the space.
     #[inline]
@@ -1006,6 +1142,16 @@ impl Ui {
     /// Shortcut for `add(Label::new(text))`
     ///
     /// See also [`Label`].
+    ///
+    /// ### Example
+    /// ```
+    /// # egui::__run_test_ui(|ui| {
+    /// use egui::{RichText, FontId, Color32};
+    /// ui.label("Normal text");
+    /// ui.label(RichText::new("Large text").font(FontId::proportional(40.0)));
+    /// ui.label(RichText::new("Red text").color(Color32::RED));
+    /// # });
+    /// ```
     #[inline]
     pub fn label(&mut self, text: impl Into<WidgetText>) -> Response {
         Label::new(text).ui(self)
@@ -1057,6 +1203,13 @@ impl Ui {
         Label::new(text.into().strong()).ui(self)
     }
 
+    /// Show text that is waker (fainter color).
+    ///
+    /// Shortcut for `ui.label(RichText::new(text).weak())`
+    pub fn weak(&mut self, text: impl Into<RichText>) -> Response {
+        Label::new(text.into().weak()).ui(self)
+    }
+
     /// Shortcut for `add(Hyperlink::new(url))`
     ///
     /// See also [`Hyperlink`].
@@ -1176,12 +1329,12 @@ impl Ui {
     pub fn radio_value<Value: PartialEq>(
         &mut self,
         current_value: &mut Value,
-        selected_value: Value,
+        alternative: Value,
         text: impl Into<WidgetText>,
     ) -> Response {
-        let mut response = self.radio(*current_value == selected_value, text);
+        let mut response = self.radio(*current_value == alternative, text);
         if response.clicked() {
-            *current_value = selected_value;
+            *current_value = alternative;
             response.mark_changed();
         }
         response
@@ -1261,9 +1414,30 @@ impl Ui {
 
     /// Show an image here with the given size.
     ///
-    /// See also [`Image`].
+    /// In order to display an image you must first acquire a [`TextureHandle`]
+    /// using [`Context::load_texture`].
+    ///
+    /// ```
+    /// struct MyImage {
+    ///     texture: Option<egui::TextureHandle>,
+    /// }
+    ///
+    /// impl MyImage {
+    ///     fn ui(&mut self, ui: &mut egui::Ui) {
+    ///         let texture: &egui::TextureHandle = self.texture.get_or_insert_with(|| {
+    ///             // Load the texture only once.
+    ///             ui.ctx().load_texture("my-image", egui::ColorImage::example())
+    ///         });
+    ///
+    ///         // Show the image:
+    ///         ui.image(texture, texture.size_vec2());
+    ///     }
+    /// }
+    /// ```
+    ///
+    /// Se also [`crate::Image`] and [`crate::ImageButton`].
     #[inline]
-    pub fn image(&mut self, texture_id: TextureId, size: impl Into<Vec2>) -> Response {
+    pub fn image(&mut self, texture_id: impl Into<TextureId>, size: impl Into<Vec2>) -> Response {
         Image::new(texture_id, size).ui(self)
     }
 }
